/*
 * Data for Atari ST to ISO Latin-1 code conversions.
 * Written by Thorsten Otto
 * based on http://www.unicode.org/Public/MAPPINGS/VENDORS/MISC/ATARIST.TXT
 */

static struct {
	unsigned char st;
	unsigned short uni;
} const known_pairs[] = {
	{ 0x80, 0xC7 },							/* C, LATIN CAPITAL LETTER C WITH CEDILLA */
	{ 0x81, 0xFC },							/* u" LATIN SMALL LETTER U WITH DIAERESIS */
	{ 0x82, 0xE9 },							/* e' LATIN SMALL LETTER E WITH ACUTE */
	{ 0x83, 0xE2 },							/* a^ LATIN SMALL LETTER A WITH CIRCUMFLEX */
	{ 0x84, 0xE4 },							/* a" LATIN SMALL LETTER A WITH DIAERESIS */
	{ 0x85, 0xE0 },							/* a` LATIN SMALL LETTER A WITH GRAVE */
	{ 0x86, 0xE5 },							/* aa LATIN SMALL LETTER A WITH RING ABOVE */
	{ 0x87, 0xE7 },							/* c, LATIN SMALL LETTER C WITH CEDILLA */
	{ 0x88, 0xEA },							/* e^ LATIN SMALL LETTER E WITH CIRCUMFLEX */
	{ 0x89, 0xEB },							/* e" LATIN SMALL LETTER E WITH DIAERESIS */
	{ 0x8a, 0xE8 },							/* e` LATIN SMALL LETTER E WITH GRAVE */
	{ 0x8b, 0xEF },							/* i" LATIN SMALL LETTER I WITH DIAERESIS */
	{ 0x8c, 0xEE },							/* i^ LATIN SMALL LETTER I WITH CIRCUMFLEX */
	{ 0x8d, 0xEC },							/* i` LATIN SMALL LETTER I WITH GRAVE */
	{ 0x8e, 0xC4 },							/* A" LATIN CAPITAL LETTER A WITH DIAERESIS */
	{ 0x8f, 0xC5 },							/* AA LATIN CAPITAL LETTER A WITH RING ABOVE */
	{ 0x90, 0xC9 },							/* E' LATIN CAPITAL LETTER E WITH ACUTE */
	{ 0x91, 0xE6 },							/* ae LATIN SMALL LETTER AE */
	{ 0x92, 0xC6 },							/* AE LATIN CAPITAL LETTER AE */
	{ 0x93, 0xF4 },							/* o^ LATIN SMALL LETTER O WITH CIRCUMFLEX */
	{ 0x94, 0xF6 },							/* o" LATIN SMALL LETTER O WITH DIAERESIS */
	{ 0x95, 0xF2 },							/* o` LATIN SMALL LETTER O WITH GRAVE */
	{ 0x96, 0xFB },							/* u^ LATIN SMALL LETTER U WITH CIRCUMFLEX */
	{ 0x97, 0xF9 },							/* u` LATIN SMALL LETTER U WITH GRAVE */
	{ 0x98, 0xFF },							/* y" LATIN SMALL LETTER Y WITH DIAERESIS */
	{ 0x99, 0xD6 },							/* O" LATIN CAPITAL LETTER O WITH DIAERESIS */
	{ 0x9a, 0xDC },							/* U" LATIN CAPITAL LETTER U WITH DIAERESIS */
	{ 0x9b, 0xA2 },							/* \cent CENT SIGN */
	{ 0x9c, 0xA3 },							/* \pound POUND SIGN */
	{ 0x9d, 0xA5 },							/* \yen YEN SIGN */
	{ 0x9e, 0xDF },							/* \ss LATIN SMALL LETTER SHARP S */
	{ 0x9f, 0x192 },						/* LATIN SMALL LETTER F WITH HOOK */
	{ 0xa0, 0xE1 },							/* a' LATIN SMALL LETTER A WITH ACUTE */
	{ 0xa1, 0xED },							/* i' LATIN SMALL LETTER I WITH ACUTE */
	{ 0xa2, 0xF3 },							/* o' LATIN SMALL LETTER O WITH ACUTE */
	{ 0xa3, 0xFA },							/* u' LATIN SMALL LETTER U WITH ACUTE */
	{ 0xa4, 0xF1 },							/* n~ LATIN SMALL LETTER N WITH TILDE */
	{ 0xa5, 0xD1 },							/* N~ LATIN CAPITAL LETTER N WITH TILDE */
	{ 0xa6, 0xAA },							/* a_ FEMININE ORDINAL INDICATOR */
	{ 0xa7, 0xBA },							/* o_ MASCULINE ORDINAL INDICATOR */
	{ 0xa8, 0xBF },							/* ?' INVERTED QUESTION MARK */
	{ 0xa9, 0x2310 },						/* REVERSED NOT SIGN */
	{ 0xaa, 0xAC },							/* \neg NOT SIGN */
	{ 0xab, 0xBD },							/* 1/2 VULGAR FRACTION ONE HALF */
	{ 0xac, 0xBC },							/* 1/4 VULGAR FRACTION ONE QUARTER */
	{ 0xad, 0xA1 },							/* !` INVERTED EXCLAMATION MARK */
	{ 0xae, 0xAB },							/* `` LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
	{ 0xaf, 0xBB },							/* '' RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
	{ 0xb0, 0xE3 },							/* a~ LATIN SMALL LETTER A WITH TILDE */
	{ 0xb1, 0xF5 },							/* o~ LATIN SMALL LETTER O WITH TILDE */
	{ 0xb2, 0xD8 },							/* O/ LATIN CAPITAL LETTER O WITH STROKE */
	{ 0xb3, 0xF8 },							/* o/ LATIN SMALL LETTER O WITH STROKE */
	{ 0xb4, 0x153 },						/* oe LATIN SMALL LIGATURE OE */
	{ 0xb5, 0x152 },						/* OE LATIN CAPITAL LIGATURE OE */
	{ 0xb6, 0xC0 },							/* A` LATIN CAPITAL LETTER A WITH GRAVE */
	{ 0xb7, 0xC3 },							/* A~ LATIN CAPITAL LETTER A WITH TILDE */
	{ 0xb8, 0xD5 },							/* O~ LATIN CAPITAL LETTER O WITH TILDE */
	{ 0xb9, 0xA8 },							/* DIAERESIS */
	{ 0xba, 0xB4 },							/* ACUTE ACCENT */
	{ 0xbb, 0x2020 },						/* DAGGER */
	{ 0xbc, 0xB6 },							/* PILCROW SIGN */
	{ 0xbd, 0xA9 },							/* COPYRIGHT SIGN */
	{ 0xbe, 0xAE },							/* REGISTERED SIGN */
	{ 0xbf, 0x2122 },						/* TRADE MARK SIGN */
	{ 0xc0, 0x133 },						/* LATIN SMALL LIGATURE IJ */
	{ 0xc1, 0x132 },						/* LATIN CAPITAL LIGATURE IJ */
	{ 0xc2, 0x5d0 },						/* HEBREW LETTER ALEF */
	{ 0xc3, 0x5d1 },						/* HEBREW LETTER BET */
	{ 0xc4, 0x5d2 },						/* HEBREW LETTER GIMEL */
	{ 0xc5, 0x5d3 },						/* HEBREW LETTER DALET */
	{ 0xc6, 0x5d4 },						/* HEBREW LETTER HE */
	{ 0xc7, 0x5d5 },						/* HEBREW LETTER VAV */
	{ 0xc8, 0x5d6 },						/* HEBREW LETTER ZAYIN */
	{ 0xc9, 0x5d7 },						/* HEBREW LETTER HET */
	{ 0xca, 0x5d8 },						/* HEBREW LETTER TET */
	{ 0xcb, 0x5d9 },						/* HEBREW LETTER YOD */
	{ 0xcc, 0x5db },						/* HEBREW LETTER KAF */
	{ 0xcd, 0x5dc },						/* HEBREW LETTER LAMED */
	{ 0xce, 0x5de },						/* HEBREW LETTER MEM */
	{ 0xcf, 0x5e0 },						/* HEBREW LETTER NUN */
	{ 0xd0, 0x5e1 },						/* HEBREW LETTER SAMEKH */
	{ 0xd1, 0x5e2 },						/* HEBREW LETTER AYIN */
	{ 0xd2, 0x5e4 },						/* HEBREW LETTER PE */
	{ 0xd3, 0x5e6 },						/* HEBREW LETTER TSADI */
	{ 0xd4, 0x5e7 },						/* HEBREW LETTER QOF */
	{ 0xd5, 0x5e8 },						/* HEBREW LETTER RESH */ 
	{ 0xd6, 0x5e9 },						/* HEBREW LETTER SHIN */
	{ 0xd7, 0x5ea },						/* HEBREW LETTER TAV */
	{ 0xd8, 0x5df },						/* HEBREW LETTER FINAL NUN */
	{ 0xd9, 0x5da },						/* HEBREW LETTER FINAL KAF */
	{ 0xda, 0x5dd },						/* HEBREW LETTER FINAL MEM */
	{ 0xdb, 0x5e3 },						/* HEBREW LETTER FINAL PE */
	{ 0xdc, 0x5e5 },						/* HEBREW LETTER FINAL TSADI */
	{ 0xDD, 0xA7 },							/* PARAGRAPH SIGN, SECTION SIGN */
	{ 0xde, 0x2227 },						/* LOGICAL AND */
	{ 0xdf, 0x221e },						/* INFINITY */
	{ 0xe0, 0x3b1 },						/* GREEK SMALL LETTER ALPHA */
	{ 0xe1, 0x3b2 },						/* GREEK SMALL LETTER BETA */
	{ 0xe2, 0x393 },						/* GREEK CAPITAL LETTER GAMMA */
	{ 0xe3, 0x3c0 },						/* GREEK SMALL LETTER PI */
	{ 0xe4, 0x3a3 },						/* GREEK CAPITAL LETTER SIGMA */
	{ 0xe5, 0x3c3 },						/* GREEK SMALL LETTER SIGMA */
	{ 0xE6, 0xB5 },							/* MICRO SIGN */
	{ 0xe7, 0x3c4 },						/* GREEK SMALL LETTER TAU */
	{ 0xe8, 0x3a6 },						/* GREEK CAPITAL LETTER PHI */
	{ 0xe9, 0x398 },						/* GREEK CAPITAL LETTER THETA */
	{ 0xea, 0x3a9 },						/* GREEK CAPITAL LETTER OMEGA */
	{ 0xeb, 0x3b4 },						/* GREEK SMALL LETTER DELTA */
	{ 0xec, 0x222e },						/* CONTOUR INTEGRAL */
	{ 0xed, 0x3c6 },						/* GREEK SMALL LETTER PHI */
	{ 0xee, 0x2208 },						/* ELEMENT OF SIGN */
	{ 0xef, 0x2229 },						/* INTERSECTION */
	{ 0xf0, 0x2261 },						/* IDENTICAL TO */
	{ 0xF1, 0xB1 },							/* +- PLUS-MINUS SIGN */
	{ 0xf2, 0x22b5 },						/* >= GREATER-THAN OR EQUAL TO */
	{ 0xf3, 0x2264 },						/* <= LESS-THAN OR EQUAL TO */
	{ 0xf4, 0x2320 },						/* TOP HALF INTEGRAL */
	{ 0xf5, 0x2321 },						/* BOTTOM HALF INTEGRAL */
	{ 0xF6, 0xF7 },							/* \div DIVISION SIGN */
	{ 0xf7, 0x2248 },						/* ALMOST EQUAL TO */
	{ 0xF8, 0xB0 },							/* \deg DEGREE SIGN */
	{ 0xf9, 0x2219 },						/* BULLET OPERATOR */
	{ 0xFA, 0xB7 },							/* \cdot MIDDLE DOT */
	{ 0xfb, 0x221a },						/* SQUARE ROOT */
	{ 0xfc, 0x207f },						/* SUPERSCRIPT LATIN SMALL LETTER N */
	{ 0xFD, 0xB2 },							/* ^2 SUPERSCRIPT TWO */
	{ 0xFE, 0xB3 },							/* ^3 SUPERSCRIPT THREE */
	{ 0xff, 0xaf }							/* MACRON */
};

#define NUMBER_OF_PAIRS (sizeof (known_pairs) / sizeof (known_pairs[0]))

